#include "Julia_Set.h"

/***
	This is the implimentation of the Julia Set Class
***/

Julia_Set::Julia_Set(int windowSize[],Complex seed,int iters)
{
	this->windowSize[0] = windowSize[0];
	this->windowSize[1] = windowSize[1];
	this->seed = seed;
	this->iters = iters;
	this->rangeX[0] = -2.0;
	this->rangeX[1] = 2.0;
	this->rangeY[0] = -2.0;
	this->rangeY[1] = 2.0;
	this->prec[0] = (rangeX[1]-rangeX[0])/windowSize[0];
	this->prec[1] = (rangeY[1]-rangeY[0])/windowSize[1];
}

Julia_Set::Julia_Set(int windowSize[],double rangeX[],double rangeY[],Complex seed,int iters)
{
	this->windowSize[0] = windowSize[0];
	this->windowSize[1] = windowSize[1];
	this->seed = seed;
	this->iters = iters;
	this->rangeX[0] = rangeX[0];
	this->rangeX[1] = rangeX[1];
	this->rangeY[0] = rangeY[0];
	this->rangeY[1] = rangeY[1];
	this->prec[0] = (rangeX[1]-rangeX[0])/windowSize[0];
	this->prec[1] = (rangeY[1]-rangeX[0])/windowSize[0];
}

int Julia_Set::escapeTest(Complex* c,const int iter)
{
	Complex seed = Complex(0.0,0.0);
	int i = 0;

	for(i;i<iter;i++)
	{
		if(pow(seed.getMag(),2) > 4) return i;
		seed = seed*seed;
		seed = seed+*c;
	}
	return -1;
}

PyObject* Julia_Set::calculate()
{
	double i = this->rangeX[0];
	double j = this->rangeY[0];
	Complex accum = this->seed;
	PyObject* map = PyTuple_New(this->windowSize[0]);
	PyObject* submap;

	for(j; j < this->rangeY[1]; j+=this->prec[1])
	{
		i = rangeX[0];
		int pos = (int)((j-this->rangeY[0])/(this->prec[1]));
		submap = PyTuple_New(this->windowSize[1]);
		for(i; i < this->rangeX[1]; i+=this->prec[0])
		{
			int pos = (int)((i-this->rangeX[0])/(this->prec[0]));
			Complex* test = new Complex(i,j);
			PyTuple_SET_ITEM(submap,pos,Py_BuildValue("i",this->escapeTest(test,this->iters)));
		}
		PyTuple_SET_ITEM(map,pos,submap);
	}

	return Py_BuildValue("O",map);
}

/** Implementation of the mutator methods. **/
void Julia_Set::setWindowSize(int windowSize[])
{
	this->windowSize[0] = windowSize[0];
	this->windowSize[1] = windowSize[1];
}

void Julia_Set::setRangeX(double rangeX[])
{
	this->rangeX[0] = rangeX[0];
	this->rangeX[1] = rangeX[1];
}

void Julia_Set::setRangeY(double rangeY[])
{
	this->rangeY[0] = rangeY[0];
	this->rangeY[1] = rangeY[1];
}

void Julia_Set::setSeed(Complex s)
{
	this->seed = s;
}

void Julia_Set::setIters(int i)
{
	this->iters = i;
}

/** Implementation of the accessor methods. **/
int* Julia_Set::getWindowSize()	{ return this->windowSize; }
double* Julia_Set::getRangeX()	{ return this->rangeX; }
double* Julia_Set::getRangeY()	{ return this->rangeY; }
Complex Julia_Set::getSeed()		{ return this->seed; }
int Julia_Set::getIters()			{ return this->iters; }
