#include "Complex.h"

/***
	 This is a class that does all of the dirty work
	 for a Julia Set.  It takes a seed, and a number
	 of iterations.  It generates an "escape map".
	 This escape map is a 2D array that holds the number
	 at which each value escaped (or -1 for non-escaping).
***/

class Julia_Set
{
	private:
		int iters;
		Complex seed;
		int windowSize[2];	// a representation of X,Y pixel size
		double rangeX[2];	// the real range of the graph
		double rangeY[2];	// the imaginary range of the graph
		double prec[2];		// this is the conversion factor between pixels and coordinates
	public:
		Julia_Set(int windowSize[],Complex seed, int iters);
		Julia_Set(int windowSize[],double rangeX[],double rangeY[],Complex seed,int iters);
		PyObject* calculate();
		int escapeTest(Complex* c,const int iters);
		/// Mutators
		void setWindowSize(int windowSize[]);
		void setRangeX(double rangeX[]);
		void setRangeY(double rangeY[]);
		void setSeed(Complex s);
		void setIters(int i);
		/// Accessors
		int* getWindowSize(void);
		double* getRangeX(void);
		double* getRangeY(void);
		Complex getSeed(void);
		int getIters(void);
};
